<?php

/** construit une structure contenant l'ensemble des scripts applications pour filtrage ultérieur
 *  les scripts peuvent être sous  deux formes : 
 *  index json  + script
 *  script défini par son nom (obsolète)
 *  
 * @param array $config
 * @return array $scripts
 */
function read_application_scripts($config)
{
    $scripts = [];
    $paths = [
        'package' => "/usr/share/sambaedu/applications/",
        'local' => "/etc/sambaedu/applications/"
    ];
    foreach ($paths as $l => $path) {
        $apps = scandir($path, SCANDIR_SORT_DESCENDING);
        foreach ($apps as $app) {
            if ($app != ".." && $app != "." && is_dir($path . $app)) {
                // index des scripts présents dans le dossier
                if (file_exists($path . $app . "/scripts.json")) {
                    $index = json_decode(file_get_contents($path . $app . "/scripts.json"), true);
                    foreach ($index as $i) {
                        $i['type'] = $l;
                        $i['app'] = $app;
                        $i['path'] = $path . $app . "/";
                        $i['includes'] = $i['includes'] ?? [];
                        $i['excludes'] = $i['excludes'] ?? [];
                        $i['applications'] = $i['applications'] ?? [];
                        $i['context'] = $i['context'] ?? ""; // once, server, system
                        if (empty($i['interpreter'])) {
                            if ($i['os'] == "windows") {
                                $i['interpreter'] = "cmd";
                            } else {
                                $i['interpreter'] = "bash";
                            }
                        }
                        if (file_exists($path . $app . "/" . $i['file'])) {
                            $i['script'] = file($path . $app . "/" . $i['file']);
                            $scripts[] = $i;
                        }
                    }
                }
                // redirections de dossiers pour des applications
                if (file_exists($path . $app . "/redirects.json")) {
                    $index = json_decode(file_get_contents($path . $app . "/redirects.json"), true);
                    foreach ($index as $appli => $i) {
                        $i['type'] = $l;
                        $i['app'] = $i['app'] ?? $appli; // si on veut un comportement différent selon les groupes/parcs, on peut définir plusieurs redirections pour une application
                        $i['action'] = "logon";
                        $i['path'] = $path . $app . "/";
                        $i['includes'] = $i['includes'] ?? [];
                        $i['excludes'] = $i['excludes'] ?? [];
                        $i['applications'] = $i['applications'] ?? [];
                        $i['context'] = "";
                        $i['os'] = "windows"; // les redirections sont uniquement pour windows pour le moment
                        $i['interpreter'] = "redirects";
                        // on remonte les redirections en premier dans l'ordre de traitement
                        array_unshift($scripts, $i);
                        // elevation des droits pour les opérations mklink /d
                        $i['context'] = "system";
                        array_unshift($scripts, $i);
                    }
                }
                // ancien mécanisme : le nom du fichier définit son usage. Un seul groupe inclus/exclus peut être défini
                if (is_dir($path . $app)) {
                    foreach (scandir($path . $app) as $file) {
                        $m = $mm = [];
                        // scripts
                        if (preg_match("/^(.*)(@(.*))?\.(windows|linux)$/U", $file, $m)) {
                            $os = $m[4];
                            if ($os == "windows") {
                                $interpreter = "cmd";
                            } else {
                                $interpreter = "bash";
                            }
                            $a = explode("-", $m[1]);
                            $action = $a[0];
                            $context = $a[1] ?? "";
                            if (! empty($m[3])) {
                                if (preg_match("/^-(.*)$/", $m[3], $mm)) {
                                    $exclude = [
                                        $mm[1]
                                    ];
                                    $include = [];
                                } else {
                                    $include = [
                                        $m[3]
                                    ];
                                    $exclude = [];
                                }
                            } else {
                                $include = $exclude = [];
                            }
                            $scripts[] = [
                                'type' => $l,
                                'app' => $app,
                                'file' => $file,
                                'path' => $path . $app . "/",
                                'action' => $action,
                                'context' => $context,
                                'excludes' => $exclude,
                                'includes' => $include,
                                'os' => $os,
                                'interpreter' => $interpreter,
                                'script' => file($path . $app . "/" . $file)
                            ];
                            // packages apt
                        } elseif (preg_match("/^packages(@(.*))?\.list$/U", $file, $m)) {
                            if (! empty($m[2])) {
                                if (preg_match("/^-(.*)$/", $m[2], $mm)) {
                                    $exclude = [
                                        $mm[1]
                                    ];
                                    $include = [];
                                } else {
                                    $include = [
                                        $m[2]
                                    ];
                                    $exclude = [];
                                }
                            } else {
                                $include = $exclude = [];
                            }
                            $scripts[] = [
                                'type' => $l,
                                'app' => $app,
                                'file' => $file,
                                'path' => $path . $app . "/",
                                'action' => "startup",
                                'context' => $context,
                                'excludes' => $exclude,
                                'includes' => $include,
                                'os' => "linux",
                                'interpreter' => "apt",
                                'script' => implode(" ", array_map('rtrim', file($path . $app . "/packages.list"))) . " "
                            ];
                        }
                    }
                }
            }
        }
    }
    file_put_contents("/tmp/scripts.json", json_encode($scripts, JSON_PRETTY_PRINT));
    return $scripts;
}

/**
 * genere les scripts applications
 * la génération sefait en une fois, les différents index sot peuplés si nécessaire en fonction des requetes dans $info
 * les scripts sont mis en cache pour être rappelés si besoin.
 *
 * @param unknown $config
 * @param array $info
 * @param array $scripts
 * @return tableau idexé par interpréteur des scripts
 */
function make_application_scripts($config, array $info, array $scripts)
{
    $texts = apcu_fetch("scripts." . $info['id']);
    if (empty($texts)) {
        $packages = "";
        $out = header_scripts($config, $info);
        // scripts scpécifiques prédéfinis
        switch ($info['context']) {
            case "system":
                add_scripts($info, "local_admin", $out, local_admin_scripts($config, $info));
                break;
            case "":
                add_scripts($info, "sudo", $out, sudo_scripts($info));
                break;
        }
        // scripts applications les locaux s'appliquent en dernier et gagnent
        foreach ($scripts as $script) {
            $include = $info['context'] != "system" || ! empty($info['userprofile']);
            $include &= empty($script['includes']) || count(array_intersect(array_map('strtolower', $script['includes']), array_map('strtolower', $info['list']))) > 0;
            $include &= count(array_intersect(array_map('strtolower', $script['excludes']), array_map('strtolower', $info['list']))) == 0;
            if ($script['action'] == $info['action'] && $script['os'] == $info['os'] && $include) {
                if ($script['context'] == "server") {
                    // scripts côté serveur générés a part
                    $out['server'] = array_merge($out['server'], $script['script']);
                } else {
                    switch ($script['action']) {
                        case "startup":
                            // traitement spécifique selon le contexte du script
                            switch ($script['context']) {
                                case "once":
                                    // les once sont à lancer au startup uniquement
                                    add_scripts($info, $script['app'], $out, once_scripts($script));
                                    break;
                                default:
                                    add_scripts($info, $script['app'], $out, $script);
                            }
                            if ($script['interpreter'] == "apt") {
                                $packages .= $script['script'];
                            }
                            break;
                        case "logon":
                            if ($info['context'] == $script['context']) {
                                // traitement spécifique selon le contexte du script
                                if ($script['interpreter'] == "redirects") {
                                    add_scripts($info, "redirect-" . $script['app'], $out, redirect_scripts($config, $info, $script));
                                } else {
                                    switch ($script['context']) {
                                        default:
                                            add_scripts($info, $script['app'], $out, $script);
                                    }
                                }
                            }
                            break;
                        case "logoff":
                            if ($info['context'] == $script['context']) {
                                // traitement spécifique selon le contexte du script
                                switch ($script['context']) {
                                    default:
                                        add_scripts($info, $script['app'], $out, $script);
                                }
                            }
                            break;
                        case "shutdown":
                            add_scripts($info, $script['app'], $out, $script);
                            break;
                        case "wpkg":
                            add_scripts($info, "wpkg_" . $script['app'], $out, wpkg_scripts($config, $info, $script));
                            break;
                    }
                }
            }
        }
        switch ($info['action']) {
            case "startup":
                add_scripts($info, "apt", $out, apt_scripts($info, $packages));
                break;
        }
        powershell_scripts($config, $info, $out);
        // on force des valeurs pour les substitutions eventuelles dans les scripts
        // $config['machine'] = $info['machine']['cn'];
        if (empty($config['cloud_perso_name'])) {
            $config['cloud_perso_name'] = "Mes Documents";
        }
        foreach ($out as $i => $s) {
            $s = array_merge($s, footer_scripts($config, $info)[$i]);
            write_param($config, $s);
            $texts[$i] = implode("", $s);
        }
        $tmpfile = sys_get_temp_dir() . "/applications-server-" . $info['id'];
        if (file_put_contents($tmpfile, $texts['server'])) {
            exec("chmod +x " . $tmpfile);
            exec("sudo " . $tmpfile);
            unlink($tmpfile);
        }
        apcu_store("scripts." . $info['id'], $texts);
    }
    return $texts;
}

/**
 * ajoute un script à l'index $interpreter avec une ligne de séparation adaptée
 *
 * @param unknown $info
 *            : info du contexte d'appel
 * @param unknown $name
 *            : nom du commentaire
 * @param unknown $out
 *            : scripts de sortie, indexé par interpreteur
 * @param unknown $script
 *            : scripts à ajouter, 'interprter' et 'script' sont utilisés
 */
function add_scripts($info, $name, &$out, $script)
{
    $separator = [
        'cmd' => [
            "\r\nREM script [" . $name . "]\r\n"
        ],
        'bash' => [
            "\n# script[" . $name . "]\n"
        ],
        'powershell' => [
            "\r\n# script[" . $name . "]\r\n"
        ]
    ];
    if (! empty($script['script']) && is_array($script['script'])) {
        $out[$script['interpreter']] = array_merge($out[$script['interpreter']], $separator[$script['interpreter']], $script['script']);
    }
}

/**
 * entête des scripts
 *
 * @param unknown $config
 * @param unknown $info
 *            : infos du contexte d'appel
 * @return unknown tableau entête avec les index correspondant aux interpréteurs
 */
function header_scripts($config, $info)
{
    $header = [
        'cmd' => [],
        'powershell' => [],
        'bash' => [],
        'server' => [
            "#!/bin/bash\n"
        ]
    ];
    if ($info['os'] == "windows" && $info['action'] == "startup") {
        $domainsid = exec("sudo net getdomainsid | grep domain | cut -d' ' -f6");
        $header['cmd'] = [
            "REM cmd\r
REM  
REM script de configuration des applications windows pour " . $info['machine']['cn'] . "\r
SET DOMAINSID=" . $domainsid . "\r
SET TAG=" . $info['salle'] . "," . $config['UAI'] . "\r
FOR /f \"skip=1 delims=\" %%s IN ('\"wmic nic where NetEnabled=true get speed\"') DO (\r
    SET SPEED=%%s\r
    GOTO speed\r
)\r
:speed\r
FOR /f \"skip=1 delims=$\" %%f IN ('\"wmic path win32_computersystemproduct get uuid\"') DO (\r
    SET UUID=%%f\r
    GOTO uuid\r
)\r
:uuid\r
SET id=" . $info['id'] . "\r
IF [%SE4FS%]==[] (\r
    SET SE4FS=" . $config['se4fs_name'] . "\r
    SETX SE4FS " . $config['se4fs_name'] . " /m\r
)\r\n"
        ];
    } elseif ($info['interpreter'] == "powershell") {
        $header['powershell'] = [
            "# script powershell " . $info['action'] . "\r\n"
        ];
    } else {
        $header['cmd'] = [
            "REM cmd\r
REM " . $info['action'] . " pour " . $info['machine']['cn'] . " et " . $info['user']['cn'] . " \r
REM script de configuration des applications windows profile=" . $info['userprofile'] . "\r
SET id=" . $info['id'] . "\r
IF [%SE4FS%]==[] (\r
    SET SE4FS=" . $config['se4fs_name'] . "\r
)\r\n"
        ];
        $header['bash'] = [
            "#!/bin/bash\n#" . $info['action'] . "\n# script de configuration des applications Linux\nid=" . $info['id'] . "\n"
        ];
    }
    return $header;
}

/**
 * fin des scripts
 *
 * @param unknown $config
 * @param unknown $info
 *            : infos du contexte d'appel
 * @return unknown fin de script correspondant à l'interpréteur appelé avec l'index correspondant
 */
function footer_scripts($config, $info)
{
    $footer = [
        'cmd' => [
            "\r\ncurl -F \"os=" . $info['os'] . "\" -F \"uuid=%UUID%\" -F \"speed=%SPEED%\" -F \"id=" . $info['id'] . "\"  -F \"ret=0\"  \"http://" . $config['se4fs_name'] . "/gpo/applications.php\">NUL\r
exit /b\r\n"
        ],
        'bash' => [
            "\n/usr/bin/curl -s -F \"id=" . $info['id'] . "\" -F \"action=" . $info['action'] . "\" -F \"os=" . $info['os'] . "\" -F \"ret=0\" http://" . $config['se4fs_name'] . "." . $config['domain'] . "/gpo/applications.php\n"
        ],
        'powershell' => [
            "# fin du script powershell\r\n"
        ],
        'server' => [
            "# fin du script server\n"
        ]
    ];
    return $footer;
}

/**
 * génère si besoin le dl et l'appel du script powershell
 *
 * @param unknown $config
 * @param unknown $info
 * @param unknown $out
 */
function powershell_scripts($config, $info, &$out)
{
    if (! empty($out['powershell'])) {
        $out['cmd'] = array_merge($out['cmd'], [
            "REM DL et exec powershell\r
curl -o \"%TEMP%\\" . $info['id'] . ".ps1\" -F \"interpreter=powershell\" -F \"id=" . $info['id'] . "\"  \"http://" . $config['se4fs_name'] . "/gpo/applications.php\">NUL\r
if exist \"%TEMP%\\%id%.ps1\" (powershell.exe -NoProfile -ExecutionPolicy Bypass -File \"%TEMP%\\%id%.ps1\")\r\n"
        ]);
    }
}

/**
 * fonctions générant les scripts spécifiques :
 *
 * @param unknown $config
 * @param unknown $info
 *            : infos du contexte d'appel
 * @return array tableau [ 'interpreter' => "shell", 'script' =>[instructions]]
 */
function once_scripts($script)
{
    $out['interpreter'] = $script['interpreter'];
    $name = $script['app'] . "-" . $script['file'] . "-" . $script['interpreter'];
    $md5 = md5_file($name);

    switch ($script['interpreter']) {
        case "bash":
            $out['script'] = [
                "[ -f \"/etc/sambaedu/applications/" . $name . ".md5\" ] && local_md5=\$(cat /etc/sambaedu/applications/" . $name . ".md5)\n",
                "if [ \"\${local_md5}\" != \"$md5\" ] ; then\n",
                "    echo \"" . $md5 . "\">\"/etc/sambaedu/applications/" . $name . ".md5\"\n",
                "fi\n"
            ];
            break;
        case "cmd":
            $out['script'] = [
                "if exist  \"%windir%\\" . $name . ".md5\" (for /F \"usebackq delims=\"  %%A in  (\"%windir%\\" . $name . ".md5\") do (set \"local_md5=%%A\"))\r\n",
                "if [%local_md5%] NEQ [$md5] (\r\n",
                "    echo " . $md5 . ">%windir%\\" . $name . ".md5\r\n",
                ")\r\n"
            ];
            break;
        case "powershell":
            $out['script'] = [
                "if (Test-Path \"\$env:windir\\" . $name . ".md5\") {\n",
                "    \$local_md5 = Get-Content -Path \"\$env:windir\\" . $name . ".md5\"\n",
                "}\n",
                "if (\$local_md5 -ne \"" . $md5 . "\") {\n",
                "    \"" . $md5 . "\" | Set-Content -Path \"\$env:windir\\" . $name . ".md5\"\n",
                "}\n"
            ];
    }
    return $out;
}

/**
 * genere les cmd pour la mise en place des redirections du profile
 *
 * 3 cas possibles
 * Local->Roaming : les préférences sont remises dans le profil itinérant (cela peut grossir, copie des données à l'ouvrture de session). La jonction est fait a chaque fois.
 * Local->Roaming->Server : les préférences Local sont accédées depuis le serveur : persistence des données. on fait un double redirect en passant par roaming afin de faire persister le link roaming->server qui doit être créé en system
 * Roaming->Server : le profil itinérant est accédé depuis le serveur : pas de copie des données, utile pour les applis avec des "gros" profils itinérants. Le lien persiste, il est fait juste une fois
 *
 * @param unknown $config
 * @param unknown $info
 * @param unknown $script
 * @return $out
 */
function redirect_scripts($config, $info, $script)
{
    $out = "";
    if (! empty($script['link']) && in_array(strtolower($script['app']), array_map('strtolower', $info['liste_applications']))) {
        if (! empty($script['server'])) {
            if (empty($info['context'])) {
                $ps = explode("\\", $script['server']);
                if (count($ps) > 1) {
                    // on créee la destination array_pop($ps);
                    $d = "";
                    foreach ($ps as $p) {
                        $d .= "\\" . $p;
                        $out .= "IF NOT EXIST \"\\\\%SE4FS%\\users\\" . $info['user']['cn'] . $d . "\" (MD \"\\\\%SE4FS%\\users\\" . $info['user']['cn'] . $d . "\")\r\n";
                    }
                }
                $ps = explode("\\", $script['link']);
                if (count($ps) > 1) {
                    array_pop($ps);
                    $d = "";
                    foreach ($ps as $p) {
                        $d .= "\\" . $p;
                        $out .= "IF NOT EXIST \"" . $info['userprofile'] . $d . "\" (MD \"" . $info['userprofile'] . $d . "\")\r\n";
                    }
                }
                if (! empty($script['dest'])) {
                    // jonction de local vers un dossier dans Roaming
                    $ps = explode("\\", $script['dest']);
                    if (count($ps) > 1) {
                        array_pop($ps);
                        $d = "";
                        foreach ($ps as $p) {
                            $d .= "\\" . $p;
                            $out .= "IF NOT EXIST \"" . $info['userprofile'] . $d . "\" (MD \"" . $info['userprofile'] . $d . "\")\r\n";
                        }
                    }
                    $out .= "IF EXIST \"" . $info['userprofile'] . "\\" . $script['link'] . "\" (RD /Q \"" . $info['userprofile'] . "\\" . $script['link'] . "\")\r\n";
                    $out .= "IF NOT EXIST \"" . $info['userprofile'] . "\\" . $script['link'] . "\" (MKLINK /J \"" . $info['userprofile'] . "\\" . $script['link'] . "\"  \"" . $info['userprofile'] . "\\" . $script['dest'] . "\")\r\n";
                }
            } else {
                // logon-system
                // lien vers le dossier du serveur
                if (! empty($script['dest'])) {
                    $out .= "IF NOT EXIST \"" . $info['userprofile'] . "\\" . $script['dest'] . "\" (MKLINK /D \"" . $info['userprofile'] . "\\" . $script['dest'] . "\" \"\\\\%SE4FS%\\users\\" . $info['user']['cn'] . "\\" . $script['server'] . "\")\r\n";
                } else {
                    $out .= "IF NOT EXIST \"" . $info['userprofile'] . "\\" . $script['link'] . "\" (MKLINK /D \"" . $info['userprofile'] . "\\" . $script['link'] . "\" \"\\\\%SE4FS%\\users\\" . $info['user']['cn'] . "\\" . $script['server'] . "\")\r\n";
                }
            }
        } else {
            // jonction Local -> Roaming
            if (empty($info['context'])) {
                $ps = explode("\\", $script['dest']);
                if (count($ps) > 1) {
                    array_pop($ps);
                    $d = "";
                    foreach ($ps as $p) {
                        $d .= "\\" . $p;
                        $out .= "IF NOT EXIST \"" . $info['userprofile'] . $d . "\" (MD \"" . $info['userprofile'] . $d . "\")\r\n";
                    }
                }
                $out .= "IF NOT EXIST \"" . $info['userprofile'] . "\\" . $script['dest'] . "\" (MD \"" . $info['userprofile'] . "\\" . $script['dest'] . "\")\r\n";
                $ps = explode("\\", $script['link']);
                if (count($ps) > 1) {
                    array_pop($ps);
                    $d = "";
                    foreach ($ps as $p) {
                        $d .= "\\" . $p;
                        $out .= "IF NOT EXIST \"" . $info['userprofile'] . $d . "\" (MD \"" . $info['userprofile'] . $d . "\")\r\n";
                    }
                }
                $out .= "IF NOT EXIST \"" . $info['userprofile'] . "\\" . $script['link'] . "\" (MKLINK /J \"" . $info['userprofile'] . "\\" . $script['link'] . "\"  \"" . $info['userprofile'] . "\\" . $script['dest'] . "\")\r\n";
            }
        }
    }
    return [
        'interpreter' => "cmd",
        'script' => [
            $out
        ]
    ];
}

function sudo_scripts($info)
{
    if ($info['os'] == "linux" && ($info['action'] == "logon" || $info['action'] == "logoff")) {
        $script = [
            "sudo /usr/share/sambaedu/scripts/system_script " . $info['action'] . "-system\n"
        ];
    } else {
        $script = [];
    }
    return [
        'interpreter' => "bash",
        'script' => $script
    ];
}

function wpkg_scripts($config, $info, $script)
{
    if ($script['app'] == $info['application'] && in_array(strtolower($script['app']), $info['liste_applications'])) {
        $out = $script['script'];
    } else {
        $out = [];
    }
    return [
        'interpreter' => $info['interpreter'],
        'script' => $out
    ];
}

function apt_scripts($info, $packages)
{
    if (! empty($packages) && $info['os'] == "linux") {
        $out = [
            "local_packages=\"" . trim($packages) . "\"\n"
        ];
    } else {
        $out = [];
    }
    return [
        'interpreter' => "bash",
        'script' => $out
    ];
}

function local_admin_scripts($config, $info)
{
    $script = "";
    if ($info['os'] == "windows") {
        $interpreter = "cmd";
        // if (preg_match("/^" . $info['machine']['cn'] . "/i", $info['user']['cn'])) {
        if (empty($info['userprofile'])) {
            // le script est appelé avec SYSTEM, et donc user=machine$
            switch ($info['action']) {
                case "logon":

                    $script .= "rem on doit boucler pour attendre que l'utilisateur soit bien connecte\r
:user\r
FOR /f \"tokens=2 delims=\\ \" %%a in ('query session ^| findstr /c:\"Acti\"') DO (\r
    SET userlogin=%%a\r
)\r
IF [%userlogin%]==[] (\r
    ping -n 1 localhost\r
    GOTO user\r
)\r
";
                    $script .= "rem on boucle jusqu'a avoir un profile\r
FOR /f \"tokens=1 delims=\" %%a in ('wmic useraccount where name^=\"%userlogin%\" get sid ^| findstr S-1-5') DO (\r
    SET SID=%%a\r
    FOR /f \"tokens=3 delims=\" %%b in ('reg query \"HKLM\\Software\\Microsoft\\Windows NT\\CurrentVersion\\ProfileList\\%SID%\" /v ProfileImagePath ^| findstr ProfileImagePath') DO (\r
        SET profile=%%b\r
    )\r
    IF NOT [%profile%]==[] (\r
        SET USERPROFILE=%profile%\r
    )\r
)\r
IF [%profile%]==[] (\r
    SET USERPROFILE=%SystemDrive%\\Users\\%userlogin%\r
)\r
";
                    $script .= ":dl\r
curl.exe -o \"%TEMP%\\applications-" . $info['action'] . "-system-%userlogin%.cmd\" -F \"action=" . $info['action'] . "-system\" -F \"os=windows\" -F \"machine=%computername%\" -F \"userprofile=%USERPROFILE%\" -F \"user=%userlogin%\" \"http://" . $config['se4fs_name'] . "/gpo/applications.php\">NUL\r
if exist \"%TEMP%\\applications-" . $info['action'] . "-system-%userlogin%.cmd\" (call \"%TEMP%\\applications-" . $info['action'] . "-system-%userlogin%.cmd\")\r
";
            }
        } else {
            $interpreter = "bash";
            switch ($info['action']) {
                case "logoff":
                    $script .= "set userlogin=" . $info['user']['cn'] . "\r\n";
                    if (get_local_admin_right($config, $info['user']['cn']) != 0) {
                        $script .= "net localgroup administrateurs " . $config['samba_domain'] . "\\" . $info['user']['cn'] . " /delete\r\n";
                    }
                    break;
                case "logon":
                    if (have_right($config, SE_COMPUTER_ADMIN, $info['user']['cn']) || have_delegation($config, $info['machine']['cn'], SE_COMPUTER_ADMIN, $info['user']['cn'])) {
                        $script .= "net localgroup administrateurs " . $config['samba_domain'] . "\\" . $info['user']['cn'] . " /add\r\n";
                        $script .= "set admin=1\r\n";
                    }
            }
        }
    } else {
        if (get_local_admin_right($config, $info['user']['cn']) != 0) {
            $u = strtr($info['user']['cn'], ".", "_");
            if ($info['action'] == "logon") {
                if (have_right($config, SE_COMPUTER_ADMIN, $info['user']['cn']) || have_delegation($config, $info['machine']['cn'], SE_COMPUTER_ADMIN, $info['user']['cn'])) {
                    $script .= "echo \"" . $info['user']['cn'] . " ALL=(ALL:ALL) ALL \" > /etc/sudoers.d/$u\n";
                    $script .= "chmod 0440 /etc/sudoers.d/$u\n";
                }
            } else {
                $script .= "[ -f /etc/sudoers.d/$u ] && rm -f /etc/sudoers.d/$u\n";
            }
        }
    }
    return [
        'interpreter' => $interpreter,
        'script' => [
            $script
        ]
    ];
}

function log_application_scripts($config, $info, $ret)
{
    if ($info['action'] == "shutdown" || $info['action'] == "logoff") {
        delete_remote_connexion($config, $info['machine']['cn']);
    } elseif ($info['action'] == "startup" && ($info['os'] == "linux" || $info['os'] == "windows") && ! in_array($info['os'], get_os($config, $info['machine']))) {
        set_os($config, $info['machine']['cn'], $info['os']);
    }
    if (empty($info['machine']['iphostnumber'])) {
        $info['machine']['iphostnumber'] = $_SERVER['REMOTE_ADDR'];
    }
    // logs
    if ($info['speed'] < 100000000) {
        $s = SAMBAEDU_SLOW_NET_ERROR;
    } else {
        $s = 0;
    }
    $err = [
        'startup' => SAMBAEDU_STARTUP_APP_ERROR,
        'shutdown' => SAMBAEDU_SHUTDOWN_APP_ERROR,
        'logon' => SAMBAEDU_LOGON_APP_ERROR,
        'logoff' => SAMBAEDU_LOGOFF_APP_ERROR,
        'logon-system' => SAMBAEDU_LOGON_SYS_APP_ERROR,
        'logoff-system' => SAMBAEDU_LOGOFF_SYS_APP_ERROR,
        'wpkg' => SAMBAEDU_WPKG_ERROR
    ];
    if ($ret == "0") {
        // log de fin d'exec des scripts
        if ($info['action'] != "wpkg") {
            if (! empty($info['uuid'])) {
                register_machine_hardware($config, $info['machine'], $info['uuid']);
            }
            log_connexion($config, $info['user']['cn'], $info['machine']['cn'], $info['os'], $info['action'], "", $err[$info['action']], true);
        }
        // effacement du cache l'id
        if ($info['action'] == "shutdown" || $info['action'] == "logoff") {
            // apcu_delete($info['action'] . "." . $info['machine']['cn'] . "." . $info['machine']['cn']);
            apcu_delete("apps." . $info['id']);
            apcu_delete("scripts." . $info['id']);
        }
        // file_put_contents("/tmp/applications-$info['action']-clear.log", $machine . " " . $id . " fini. ", FILE_APPEND);
        // on ne fait rien de plus
        return false;
    } else {
        // log de début des scripts
        if ($info['action'] != "wpkg") {
            log_connexion($config, $info['user']['cn'], $info['machine']['cn'], $info['os'], $info['action'], $info['machine']['iphostnumber'], $err[$info['action']] | $s);
        }
        return true;
    }
}

function get_app_scripts_info($config)
{
    $conf = HTMLPurifier_Config::createDefault();
    $purifier = new HTMLPurifier($conf);
    $machine = $_POST['machine'] ?? $_GET['machine'] ?? "";
    $action = $_POST['action'] ?? $_GET['action'] ?? "";
    $application = $_POST['application'] ?? $_GET['application'] ?? "";
    $os = $_POST['os'] ?? $_GET['os'] ?? "windows";
    $uuid = $_POST['uuid'] ?? $_GET['uuid'] ?? "";
    $interpreter = $_POST['interpreter'] ?? $_GET['interpreter'] ?? "";
    if (empty($interpreter)) {
        if ($os == "linux") {
            $interpreter = "bash";
        } else {
            $interpreter = "cmd";
        }
    }
    $speed = $_POST['speed'] ?? $_GET['speed'] ?? 100000000;
    $uuid = trim(strtolower($uuid));
    $user = $_POST['user'] ?? $_GET['user'] ?? "";
    $id = $_POST['id'] ?? $_GET['id'] ?? "";
    $action = $purifier->purify($action);
    $os = $purifier->purify($os);
    $user = $purifier->purify($user);
    $userprofile = $_POST['userprofile'] ?? $_GET['userprofile'] ?? "";
    $machine = strtolower($purifier->purify($machine));
    // pour les ltsp
    $machine = preg_replace("/^l-/", "", $machine);
    if (empty($id)) {
        if ($user == "Debian-gdm" || $user == "root" || preg_match("/remote-/", $action)) {
            // pas de script pour les utilisateurs ci-dessus au logon/logoff donc également pour la première cnx logon-system, où user == machine$
            return [];
        }
        if ($action == "logon-system" || $action == "logoff-system") {
            if (empty($machine)) {
                // ne devrait jamais se produire, mais cela semble arriver...
                $out = "rem oups appel sans arguments !\r
exit 1\r
";
                echo $out;
                file_put_contents("/tmp/applications-$action-error-last.log", $out);
                return [];
            }
            // inutile ??
            apcu_store($action . "." . $machine . "." . $user, $action, 300);
        }

        $id = md5(strtolower($user) . strtolower($machine) . $action . $application);
    }
    $info = apcu_fetch("apps." . $id);
    if (empty($info)) {
        $context = explode("-", $action)[1] ?? "";
        $action = explode("-", $action)[0];
        $userl = $machinel = [];
        $admin = 0;
        // recherche de sinfos machine
        if (! empty($machine)) {
            if ($action == "startup") {
                $html = "";
                check_computer($config, $machine, $html);
            }
            $machinel = search_machine($config, $machine, true);
        }
        $machinel['cn'] = $machinel['cn'] ?? $machine;
        // recherche des info $user
        if ($m = preg_match("/" . $machine . "/i", $user)) {
            $userl = $machinel;
        } else {
            if (! empty($user)) {
                $userl = search_user($config, $user);
                exec("/usr/share/sambaedu/shares/share.avail/mkhome.sh " . $user);
            } else {
                $cnx_user = get_machine_status($config, $machinel['cn']);
                if (! empty($cnx_user['user'][0]['name'])) {
                    $userl = search_user($config, $cnx_user['user'][0]['name']);
                }
            }
        }
        // recherche des parcs
        $parcs = [];
        if (! empty($machinel['dn'])) {
            $salle = ldap_dn2cn(ldap_dn2oudn($machinel['dn']));
            // $parcs[] = $machine['cn'];
            if (isset($machinel['memberof'])) {
                foreach ($machinel['memberof'] as $parc) {
                    $parcs[] = ldap_dn2cn($parc);
                }
            }
            unset($parcs[$salle]);
        } else {
            $salle = "";
        }
        $groups = $groups_e = [];
        $list_u = $list_ue = $list_m = [];
        if (! $m && isset($userl['cn'])) {
            if (get_local_admin_right($config, $userl['cn']) > 0 && (have_right($config, SE_COMPUTER_ADMIN, $userl['cn']) || have_delegation($config, $machinel['cn'], SE_COMPUTER_ADMIN, $userl['cn']))) {
                $admin = 1;
            }
            foreach ($userl['memberof'] as $group) {
                $groups[] = ldap_dn2cn($group);
                $groups_e[] = ldap_dn2sam($group);
            }
            $list_u = array_unique(array_merge([
                $userl['cn']
            ], $groups));
            $list_ue = array_unique(array_merge([
                $userl['cn']
            ], $groups_e));
        }
        $list_m = array_unique(array_merge([
            $machinel['cn']
        ], [
            $salle
        ], $parcs));
        $list = array_unique(array_merge($list_u, $list_m));
        $liste_applications = [];
        if (! empty($action)) {
            if (file_exists("../includes/wpkg_lib.php")) {
                require "wpkg_lib.php";
                require "wpkg_libsql.php";
                $liste_applications = array_map('strtolower', array_column(array_column(info_poste_applications($config, $machinel['cn']), "info_app"), "id_nom_app"));
                if ($os == "windows") {
                    array_push($liste_applications, "edge");
                }
            }
        }
        $info = [
            'id' => $id,
            'action' => $action,
            'context' => $context,
            'application' => $application,
            'interpreter' => $interpreter,
            'speed' => $speed,
            'user' => $userl,
            'uuid' => $uuid,
            'userprofile' => $userprofile,
            'salle' => $salle,
            'machine' => $machinel,
            'list' => $list, // tous les groupes user et parc
            'list_u' => $list_u, // groupes
            'list_ue' => $list_ue, // groupes avec prefix
            'list_m' => $list_m, // parcs
            'liste_applications' => $liste_applications,
            'admin' => $admin,
            'os' => $os
        ];
        apcu_store("apps." . $id, $info, 1800);
    }
    return $info;
}
?>